
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace SatAMGame
{
    public enum GameState
    {
        None,
        Title,
        Level1,
        GameOver
    }

    public class GameLoop : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        Level currentLevel;
        GameState currentState = GameState.None;
        GameState nextState = GameState.Title;

        public GameLoop()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }

        protected override void Initialize()
        {
            //Init sound
            Sound.Initialize();
            base.Initialize();
        }

        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {

            }
        }

        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }

        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            //[Escape] to close game
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            //States
            if (nextState == GameState.Title)
            {
                //Init level
                currentLevel = null;
                currentLevel = new Level();
                currentLevel.AddMap(@"Maps\Sonic_logo");
                currentLevel.PlayStaticBGM("SonicSatAMTheme");

                currentState = nextState;
                nextState = GameState.None;
            }
            else if (nextState == GameState.Level1)
            {
                //Init level
                currentLevel = null;
                currentLevel = new Level(Character.Sonic);
                currentLevel.AddMap(@"Maps\Level1");
                currentLevel.Maps[0].SpriteWidth = 6400;
                //currentLevel.PlayStaticBGM("music");

                currentState = nextState;
                nextState = GameState.None;
            }

            if (currentState == GameState.Title)
            {
                //Title code
                if (Keyboard.GetState().IsKeyDown(Keys.Enter))
                {
                    currentState = GameState.None;
                    nextState = GameState.Level1;
                }
            }
            else if (currentState == GameState.Level1)
            {
                currentLevel.Input();
                currentLevel.Update(gameTime.ElapsedGameTime);
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.White);
            
            currentLevel.Render(content, graphics);           

            base.Draw(gameTime);
        }
    }
}