using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
namespace SatAMGame
{
    public enum Character
    {
        None,
        Sonic,
        Sally
    }

    class Level
    {
        List<Frame> _maps = new List<Frame>();
        List<Vector2> _mapPos = new List<Vector2>();

        Sonic _playerSonic;
        Sally _playerSally;

        List<Enemy> _enemies;
        List<GameObject> _objects;
        Character _lvlcharacter = Character.None;

        bool _scrollToCamera = true;

        Cue _bgm = null;

        #region Properties
        public List<Frame> Maps
        {
            get { return _maps; }
            set { _maps = value; }
        }

        public List<Vector2> MapPos
        {
            get { return _mapPos; }
            set { _mapPos = value; }
        }

        public Sonic PlayerSonic
        {
            get { return _playerSonic; }
            set { _playerSonic = value; }
        }

        public Sally PlayerSally
        {
            get { return _playerSally; }
            set { _playerSally = value; }
        }

        public List<Enemy> LevelEnemies
        {
            get { return _enemies; }
            set { _enemies = value; }
        }

        public List<GameObject> LevelObjects
        {
            get { return _objects; }
            set { _objects = value; }
        }

        public Character LevelCharacter
        {
            get { return _lvlcharacter; }
            set { _lvlcharacter = value; }
        }

        public Cue BGM
        {
            get { return _bgm; }
            set { _bgm = value; }
        }

        public bool ScrollToCamera
        {
            get { return _scrollToCamera; }
            set { _scrollToCamera = value; }
        }
        #endregion

        public Level()
        {
            _playerSonic = new Sonic();
            _playerSally = new Sally();

            _enemies = new List<Enemy>();
            _objects = new List<GameObject>();
        }

        public Level(Character lchar)
        {
            LevelCharacter = lchar;

            _playerSonic = new Sonic();
            _playerSally = new Sally();

            _enemies = new List<Enemy>();
            _objects = new List<GameObject>();
        }

        public void AddMap(string texture)
        {
            Maps.Add(new Frame(texture));
            MapPos.Add(Vector2.Zero);
        }

        public void PlayStaticBGM(string sound)
        {
            Sound.Play(sound);
        }

        public void PlayBGM()
        {
            Sound.Play(BGM);
        }

        public void ScrollMap()
        {
            if (ScrollToCamera)
            {
                int spriteWidth;
                int spriteHeight;

                for(int i = 0; i < MapPos.Count; i++)
                {
                    if (LevelCharacter == Character.Sonic)
                    {
                        spriteWidth = PlayerSonic.CurrentAnimation.AnimationFrames[PlayerSonic.CurrentAnimation.CurrentFrame].SpriteWidth;
                        spriteHeight = PlayerSonic.CurrentAnimation.AnimationFrames[PlayerSonic.CurrentAnimation.CurrentFrame].SpriteHeight;

                        if (PlayerSonic.Position.X > 400 - spriteWidth / 2 && PlayerSonic.Position.X < Maps[i].SpriteWidth - 400 - spriteWidth / 2)
                        {
                            if (!PlayerSonic.CamLocked && PlayerSonic.CurrentState == Player.PlayerState.Jumping)
                                PlayerSonic._velocity.X = 0f;

                            PlayerSonic.CamLocked = true;

                            MapPos[i] = Vector2.Zero - new Vector2(PlayerSonic.Position.X, 300 - spriteHeight / 2);
                            MapPos[i] = Vector2.Add(new Vector2(400 - spriteWidth / 2, 300 - spriteHeight / 2), MapPos[i]);

                            PlayerSonic.RelativePos = new Vector2(400 - spriteWidth / 2, PlayerSonic.RelativePos.Y);
                        }
                        else
                        {
                            PlayerSonic.CamLocked = false;
                        }
                    }
                    else if (LevelCharacter == Character.Sally)
                    {
                        spriteWidth = PlayerSally.CurrentAnimation.AnimationFrames[PlayerSally.CurrentAnimation.CurrentFrame].SpriteWidth;
                        spriteHeight = PlayerSally.CurrentAnimation.AnimationFrames[PlayerSally.CurrentAnimation.CurrentFrame].SpriteHeight;

                        if (PlayerSally.Position.X > 400 - spriteWidth / 2 && PlayerSally.Position.X < Maps[i].SpriteWidth - 400 - spriteWidth / 2)
                        {
                            MapPos[i] = Vector2.Zero - PlayerSally.Position;
                            MapPos[i] = Vector2.Add(new Vector2(400 - spriteWidth / 2, 300 - spriteHeight / 2), MapPos[i]);

                            PlayerSally.CamLocked = true;
                        }
                        else
                        {
                            PlayerSally.CamLocked = false;
                        }
                    }
                    else
                    {
                        MapPos[i] = Vector2.Zero;
                    }
                }
            }
        }

        public void Input()
        {
            if (LevelCharacter == Character.Sonic)
                PlayerSonic.Input();
            else if (LevelCharacter == Character.Sally)
                PlayerSally.Input();
        }

        public void Update(TimeSpan ElapsedTime)
        {
            if (LevelCharacter == Character.Sonic)
            {
                PlayerSonic.Update(ElapsedTime);
            }
            else if (LevelCharacter == Character.Sally)
            {
                PlayerSally.Update(ElapsedTime);
            }

            ScrollMap();
            Sound.Update();
        }

        public void Render(ContentManager content, GraphicsDeviceManager graphics)
        {
            SpriteBatch levelMap = new SpriteBatch(graphics.GraphicsDevice);
            levelMap.Begin();

            for (int i = 0; i < Maps.Count; i++)
            {
                levelMap.Draw(content.Load<Texture2D>(Maps[i].FramePath), MapPos[i], null, Maps[i].ColorTint);
            }

            if (LevelCharacter == Character.Sonic)
                PlayerSonic.Render(content, levelMap);
            else if (LevelCharacter == Character.Sally)
                PlayerSally.Render(content, levelMap);
            //_enemies.Render(content, levelMap);
            //_objects.Render(content, levelMap);
            Effects.doEffects();

            levelMap.End();
            levelMap.Dispose();
        }
    }
}
