
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
#endregion

namespace SatAMGame
{
    public enum Direction
    {
        None,
        Left,
        Right
    }

    class Player
    {
        Vector2 _position = new Vector2(400f, 300f);
        Vector2 _relativePos = new Vector2(400f, 300f);
        public Vector2 _velocity = new Vector2();
        Vector2 _acceleration = new Vector2(0f, 1f);
        Vector2 _gravityAcceleration = new Vector2(0f, 100f);

        Rectangle _collisionRect = new Rectangle();

        //Locks player controls
        bool _locked = false;

        //Locked in the center of the viewport
        bool _camLocked = false;

        Animation _currentAnimation = new Animation();

        #region Static Animations
        Animation _idle;
        Animation _run;
        Animation _walk;
        Animation _jump;
        Animation _hurt;
        Animation _crouch;
        #endregion

        PlayerState _curstate = PlayerState.Idle;
        Direction _facingdir = Direction.Right;

        float _tvelocity = 0f;
        float _xspeed = 0f;
        float _yspeed = 0f;

        static int _gameLoopCounter = 0;

        #region Properties
        public Vector2 Position
        {
            get { return _position; }
            set { _position = value; }
        }

        public Vector2 RelativePos
        {
            get { return _relativePos; }
            set { _relativePos = value; }
        }

        public Vector2 Acceleration
        {
            get { return _acceleration; }
            set { _acceleration = value; }
        }

        public Vector2 GravityAcceleration
        {
            get { return _gravityAcceleration; }
            set { _gravityAcceleration = value; }
        }

        public Rectangle CollisionRect
        {
            get { return _collisionRect; }
            set { _collisionRect = value; }
        }

        public float TerminalVelocity
        {
            get { return _tvelocity; }
            set { _tvelocity = value; }
        }

        public PlayerState CurrentState
        {
            get { return _curstate; }
            set { _curstate = value; }
        }

        public Direction FacingDirection
        {
            get { return _facingdir; }
            set { _facingdir = value; }
        }

        public Animation CurrentAnimation
        {
            get { return _currentAnimation; }
            set { _currentAnimation = value; }
        }

        public float XSpeed
        {
            get { return _xspeed; }
            set { _xspeed = value; }
        }

        public float YSpeed
        {
            get { return _yspeed; }
            set { _yspeed = value; }
        }

        public Animation Idle_a
        {
            get { return _idle; }
            set { _idle = value; }
        }

        public Animation Walk_a
        {
            get { return _walk; }
            set { _walk = value; }
        }

        public Animation Run_a
        {
            get { return _run; }
            set { _run = value; }
        }

        public Animation Jump_a
        {
            get { return _jump; }
            set { _jump = value; }
        }

        public Animation Crouch_a
        {
            get { return _crouch; }
            set { _crouch = value; }
        }

        public Animation Hurt_a
        {
            get { return _hurt; }
            set { _hurt = value; }
        }

        public int GameLoopCounter
        {
            get { return _gameLoopCounter; }
            set { _gameLoopCounter = value; }
        }

        public bool Locked
        {
            get { return _locked; }
            set { _locked = value; }
        }

        public bool CamLocked
        {
            get { return _camLocked; }
            set { _camLocked = value; }
        }
        #endregion

        public enum PlayerState
        {
            None,
            Idle,
            Jumping,
            Moving,
            Running,
            UsingItem,
            Attacking,
            Hurt,
            Dead
        }

        public Player()
        {
            Initialize();
        }

        public void ResetCounter()
        {
            _gameLoopCounter = 0;
        }

        public void Move(Direction dir)
        {
            switch(dir)
            {
                case Direction.Right:

                    if (CurrentState == PlayerState.Idle)
                    {
                        CurrentState = PlayerState.Moving;
                        _velocity.X = XSpeed;
                    }
                    else if (CurrentState == PlayerState.Jumping)
                    {
                        _velocity.X = (float)Math.Pow((double)XSpeed, 3) * -1;
                    }
                    else
                    {
                        _velocity.X = XSpeed;
                    }

                    FacingDirection = Direction.Right;

                    break;
                case Direction.Left:

                    if (CurrentState == PlayerState.Idle)
                    {
                        CurrentState = PlayerState.Moving;
                        _velocity.X = -XSpeed;
                    }
                    else if (CurrentState == PlayerState.Jumping)
                    {
                        _velocity.X = (float)Math.Pow((double)XSpeed, 3);
                    }
                    else
                    {
                        _velocity.X = -XSpeed;
                    }

                    FacingDirection = Direction.Left;

                    break;
            }
        }

        public void Jump(TimeSpan ElapsedTime)
        {
            _velocity += Vector2.Multiply(Acceleration, (float)ElapsedTime.TotalSeconds);
            Position -= Vector2.Multiply(_velocity, (float)ElapsedTime.TotalSeconds);

            if (!CamLocked)
                RelativePos -= Vector2.Multiply(_velocity, (float)ElapsedTime.TotalSeconds);
            else
                RelativePos -= Vector2.Multiply(new Vector2(0f, _velocity.Y), (float)ElapsedTime.TotalSeconds);

            Acceleration -= GravityAcceleration;
        }

        //Helper class
        public bool Collision(Rectangle other)
        {
            return (CollisionRect.Right > other.Left &&
                    CollisionRect.Left < other.Right &&
                    CollisionRect.Bottom > other.Top &&
                    CollisionRect.Top < other.Bottom);
        }

        //Pixel perfect collision
        //public void CollisionWithSprite(Player other)
        //{
        //    if (Collision(other.CollisionRect))
        //    {
        //        uint[] bitsA = new uint[CurrentAnimation.AnimationFrames[CurrentAnimation.CurrentFrame].SpriteWidth * CurrentAnimation.AnimationFrames[CurrentAnimation.CurrentFrame].SpriteHeight];
        //        a.texture.GetData<uint>(bitsA);

        //        uint[] bitsB = new uint[other.CurrentAnimation.AnimationFrames[other.CurrentAnimation.CurrentFrame].SpriteWidth * other.CurrentAnimation.AnimationFrames[other.CurrentAnimation.CurrentFrame].SpriteHeight];
        //        b.texture.GetData<uint>(bitsB);

        //        int x1 = Math.Max(a.bounds.X, b.bounds.X);
        //        int x2 = Math.Min(a.bounds.X + a.bounds.Width, b.bounds.X + b.bounds.Width);

        //        int y1 = Math.Max(a.bounds.Y, b.bounds.Y);
        //        int y2 = Math.Min(a.bounds.Y + a.bounds.Height, b.bounds.Y + b.bounds.Height);

        //        for (int y = y1; y < y2; ++y)
        //        {
        //            for (int x = x1; x < x2; ++x)
        //            {
        //                if (((bitsA[(x - a.bounds.X) + (y - a.bounds.Y) * a.texture.Width] & 0xFF000000) >> 24) > 20 &&
        //                    ((bitsB[(x - b.bounds.X) + (y - b.bounds.Y) * b.texture.Width] & 0xFF000000) >> 24) > 20)
        //                    return true;
        //            }
        //        }

        //        return false;
        //    }
        //}

        public virtual void Input()
        {
            if (!Locked)
            {
                if (Keyboard.GetState().IsKeyDown(Keys.Left))
                {
                    Move(Direction.Left);
                }
                else if (Keyboard.GetState().IsKeyDown(Keys.Right))
                {
                    Move(Direction.Right);
                }
                else if (CurrentState != PlayerState.Jumping &&
                         CurrentState != PlayerState.Running &&
                         CurrentState != PlayerState.None)
                {
                    CurrentState = PlayerState.Idle;
                }

                if (Keyboard.GetState().IsKeyDown(Keys.LeftShift) &&
                    CurrentState != PlayerState.Jumping &&
                    CurrentState != PlayerState.Running)
                {
                    CurrentState = PlayerState.Jumping;

                    _velocity.Y = YSpeed;
                    _velocity.X = 0;
                }
            }
        }

        public virtual void Update(TimeSpan ElapsedTime)
        {
            if (CurrentState == PlayerState.Moving)
            {
                Position = Vector2.Add(Position, _velocity);

                if(!CamLocked)
                    RelativePos = Vector2.Add(RelativePos, _velocity);

                CurrentAnimation = _walk;
            }
            else if (CurrentState == PlayerState.Idle)
            {
                _velocity.X = 0;

                CurrentAnimation = _idle;
                ResetCounter();
                CurrentAnimation.ResetCounter();
            }
            else if (CurrentState == PlayerState.Jumping)
            {
                CurrentAnimation = _jump;
                Jump(ElapsedTime);
            }

            CurrentAnimation.Next();
        }

        public virtual void Render(ContentManager content, SpriteBatch spBatch)
        {
            Frame curFrame = CurrentAnimation.AnimationFrames[CurrentAnimation.CurrentFrame];

            if (FacingDirection == Direction.Right)
            {
                spBatch.Draw(content.Load<Texture2D>(curFrame.FramePath), new Rectangle((int)RelativePos.X, (int)RelativePos.Y, curFrame.SpriteWidth, curFrame.SpriteHeight), null, curFrame.ColorTint, 0, Vector2.Zero, SpriteEffects.None, 0);
            }
            else if (FacingDirection == Direction.Left)
            {
                spBatch.Draw(content.Load<Texture2D>(curFrame.FramePath), new Rectangle((int)RelativePos.X, (int)RelativePos.Y, curFrame.SpriteWidth, curFrame.SpriteHeight), null, curFrame.ColorTint, 0, Vector2.Zero, SpriteEffects.FlipHorizontally, 0);
            }
        }

        public void Initialize()
        {
            //Arbitrary initialization
        }
    }
}


