using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
using System.Text;

namespace SatAMGame
{
    class Sonic : Player
    {
        const int _timeToCharge = 35;
        const int _baseSpriteWidth = 115;
        const int _baseSpriteHeight = 180;

        SonicState SecondaryState = SonicState.None;

        #region Static Animations
        Animation _spin;
        Animation _skid;
        Animation _charge;
        Animation _invRing;
        Animation _smoke;

        string _basePath = @"Animations\Sonic\";
        #endregion

        #region Static Sounds
        Cue speed1;
        Cue speed2;
        Cue hurt;
        Cue gottaJuice;
        #endregion

        public enum SonicState
        {
            None,
            SuperSpeed,
            Skidding,
            Tripping,
            SpinDashing,
            Charging,
            PowerRing
        }

        public Sonic()
        {
            XSpeed = 5f;
            YSpeed = 400f;

            Position = new Vector2(400 - _baseSpriteWidth / 2, 300 - _baseSpriteHeight / 2);
            RelativePos = Position;

            Initialize_StaticAnimations();
        }

        //This stuff should probably utilize some sort of external configuration file
        public void Initialize_StaticAnimations()
        {
            Frame frame = new Frame();

            //-------------------
            //  Character
            //-------------------

            Idle_a = new Animation();
            Idle_a.NumberOfFrames = 1;

            //Idle--Frame 1
            frame.FramePath = _basePath + "Sonic_Idle";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Idle_a.AnimationFrames.Add(frame);

            //Refresh memory
            frame = null;
            frame = new Frame();

            //-------------------

            Walk_a = new Animation();
            Walk_a.NumberOfFrames = 4;

            //Walk--Frame 1
            frame.FrameDelay = 10;
            frame.FramePath = _basePath + "Sonic_Idle";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Walk_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Walk--Frame 2
            frame.FrameDelay = 10;
            frame.FramePath = _basePath + "Sonic_Move1";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Walk_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Walk--Frame 3
            frame.FrameDelay = 10;
            frame.FramePath = _basePath + "Sonic_Idle";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Walk_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Walk--Frame 4
            frame.FrameDelay = 10;
            frame.FramePath = _basePath + "Sonic_Move2";
            frame.HFlip = true;
            frame.Loop = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Walk_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //-------------------

            Jump_a = new Animation();
            Jump_a.NumberOfFrames = 1;

            //Jump--Frame 1
            frame.FramePath = _basePath + "Sonic_Jump";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Jump_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //-------------------

            _charge = new Animation();
            _charge.NumberOfFrames = 10;

            //Charge--Frame 1
            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Sonic_Charge1";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 2
            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Sonic_Charge2";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 3
            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Sonic_Charge3";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 4
            frame.FrameDelay = 2;
            frame.FramePath = _basePath + "Sonic_Charge4";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 5
            frame.FrameDelay = 2;
            frame.FramePath = _basePath + "Sonic_Charge5";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 6
            frame.FrameDelay = 2;
            frame.FramePath = _basePath + "Sonic_Charge1";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 7
            frame.FrameDelay = 1;
            frame.FramePath = _basePath + "Sonic_Charge2";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 8
            frame.FrameDelay = 1;
            frame.FramePath = _basePath + "Sonic_Charge3";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 9
            frame.FrameDelay = 1;
            frame.FramePath = _basePath + "Sonic_Charge4";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Charge--Frame 10
            frame.FrameDelay = 1;
            frame.FramePath = _basePath + "Sonic_Charge5";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _charge.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //-------------------

            Run_a = new Animation();
            Run_a.NumberOfFrames = 3;

            //Run--Frame 1
            frame.FrameDelay = 5;
            frame.FramePath = _basePath + "Sonic_Run1";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Run_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Run--Frame 2
            frame.FrameDelay = 5;
            frame.FramePath = _basePath + "Sonic_Run2";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Run_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //Run--Frame 3
            frame.FrameDelay = 5;
            frame.FramePath = _basePath + "Sonic_Run3";
            frame.HFlip = true;
            frame.Loop = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            Run_a.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //-------------------

            _skid = new Animation();
            _skid.NumberOfFrames = 1;

            //Skid--Frame 1
            frame.FramePath = _basePath + "Sonic_Skid";
            frame.HFlip = true;
            frame.SpriteHeight = _baseSpriteHeight;
            frame.SpriteWidth = _baseSpriteWidth;
            _skid.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            //-------------------
            //  Effects
            //-------------------

            _smoke = new Animation();
            _smoke.NumberOfFrames = 9;

            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Smoke_Start4";
            frame.HFlip = true;
            frame.Transparency = 25;
            _smoke.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Smoke_Start3";
            frame.HFlip = true;
            frame.Transparency = 50;
            _smoke.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Smoke_Start2";
            frame.HFlip = true;
            frame.Transparency = 75;
            _smoke.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Smoke_Start1";
            frame.HFlip = true;
            _smoke.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();

            frame.FrameDelay = 3;
            frame.FramePath = _basePath + "Smoke_Start0";
            frame.HFlip = true;
            _smoke.AnimationFrames.Add(frame);

            frame = null;
            frame = new Frame();
        }

        public override void Input()
        {
            if (!Locked)
            {
                if (Keyboard.GetState().IsKeyDown(Keys.Space) &&
                    SecondaryState != SonicState.SuperSpeed &&
                    CurrentState == PlayerState.Idle)
                {
                    CurrentState = PlayerState.None;
                    SecondaryState = SonicState.Charging;
                }
                else if (!Keyboard.GetState().IsKeyDown(Keys.Space) &&
                          SecondaryState == SonicState.Charging)
                {
                    CurrentAnimation.ResetAnimation();

                    CurrentState = PlayerState.Idle;
                    SecondaryState = SonicState.None;
                }

                if (Keyboard.GetState().IsKeyDown(Keys.Down) &&
                    SecondaryState == SonicState.SuperSpeed)
                {
                    CurrentAnimation = _skid;
                    SecondaryState = SonicState.Skidding;
                    Locked = true;
                }
            }

            if(SecondaryState != SonicState.Charging)
                base.Input();
        }

        public override void Update(TimeSpan ElapsedTime)
        {
            if (SecondaryState == SonicState.Charging)
            {
                CurrentAnimation = _charge;

                //Todo: Gradually increase speed of charge animation
                GameLoopCounter++;
            }

            if (SecondaryState == SonicState.Charging && GameLoopCounter == _timeToCharge)
            {
                CurrentState = PlayerState.Running;
                SecondaryState = SonicState.SuperSpeed;

                CurrentAnimation.ResetAnimation();
                CurrentAnimation = Run_a;

                ResetCounter();

                XSpeed = 25f;

                if (FacingDirection == Direction.Left)
                    _velocity.X = -XSpeed;
                else
                    _velocity.X = XSpeed;
            }

            if (CurrentState == PlayerState.Running)
            {
                Position = Vector2.Add(Position, _velocity);

                if (!CamLocked)
                    RelativePos = Vector2.Add(RelativePos, _velocity);
            }

            if (SecondaryState == SonicState.Skidding)
            {
                //Gradually decrease speed
                if (XSpeed-- == 0f)
                {
                    XSpeed = 5f;

                    Locked = false;

                    CurrentState = PlayerState.Idle;
                    SecondaryState = SonicState.None;
                }
            }

            //Map collision
            if (CurrentState == PlayerState.Jumping && RelativePos.Y > 300 - _baseSpriteHeight / 2)
            {
                Position = new Vector2(Position.X, 300 - _baseSpriteHeight / 2);
                RelativePos = new Vector2(RelativePos.X, 300 - _baseSpriteHeight / 2);

                _velocity = Vector2.Zero;
                Acceleration = new Vector2(1f, 1f);
                CurrentState = PlayerState.Idle;
            }

            base.Update(ElapsedTime);
        }

        public override void Render(ContentManager content, SpriteBatch spBatch)
        {
            //After image
            if (CurrentState == PlayerState.Running)
            {
                Color TransparentColor;

                for (int i = 0; i < 5; i++)
                {
                    TransparentColor = new Color(200, 200, 255, (byte)(255 - i * 50));

                    if (FacingDirection == Direction.Left)
                    {
                        spBatch.Draw(content.Load<Texture2D>(CurrentAnimation.AnimationFrames[CurrentAnimation.CurrentFrame].FramePath), new Rectangle((int)RelativePos.X + (20 * i), (int)RelativePos.Y, _baseSpriteWidth, _baseSpriteHeight), null, TransparentColor, 0, Vector2.Zero, SpriteEffects.FlipHorizontally, 0);
                    }
                    else if (FacingDirection == Direction.Right)
                    {
                        spBatch.Draw(content.Load<Texture2D>(CurrentAnimation.AnimationFrames[CurrentAnimation.CurrentFrame].FramePath), new Rectangle((int)RelativePos.X - (20 * i), (int)RelativePos.Y, _baseSpriteWidth, _baseSpriteHeight), null, TransparentColor, 0, Vector2.Zero, SpriteEffects.None, 0);
                    }
                }
            }

            base.Render(content, spBatch);
        }
    }
}